package eu.mhsl.minecraft.pixelblocks;

import eu.mhsl.minecraft.pixelblocks.pixelblock.PixelBlock;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.Recipe;
import org.bukkit.inventory.ShapedRecipe;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

public class PixelBlockItem {
    public static UUID unusedBlockID = UUID.fromString("98fdf0ae-c3ab-4ef7-ae25-efd518d600de");

    public static @NotNull ItemStack getBlockAsItem(@NotNull PixelBlock block) {
        return PixelBlockItem.updateBlockAsItem(block, ItemStack.of(Material.GRAY_STAINED_GLASS));
    }

    public static @NotNull ItemStack updateBlockAsItem(@NotNull PixelBlock block, @NotNull ItemStack itemStack) {
        String ownerName = Optional.ofNullable(Bukkit.getOfflinePlayer(block.ownerUUID).getName()).orElseGet(() -> block.ownerUUID.toString());

        ItemMeta meta = itemStack.getItemMeta();
        meta.itemName(Component.text(block.blockUUID.toString()));
        meta.displayName(Component.text("Pixelblock von " + ownerName));
        meta.lore(List.of(
            Component.text(ownerName + " ist der Besitzer dieses Blocks."),
            Component.text("Klicke auf den gesetzten Block, um diesen zu bearbeiten!"),
            Component.text(block.blockUUID.toString()).color(NamedTextColor.DARK_GRAY)
        ));
        meta.setEnchantmentGlintOverride(true);
        itemStack.setItemMeta(meta);

        return itemStack;
    }

    public static @NotNull ItemStack getEmptyPixelBlock() {
        ItemStack item = ItemStack.of(Material.GRAY_STAINED_GLASS);
        ItemMeta meta = item.getItemMeta();
        meta.displayName(Component.text("Leerer Pixelblock"));
        meta.lore(List.of(
            Component.text("Der erste Spieler, welcher den Block platziert wird der Besitzer des Blocks."),
            Component.text("Klicke auf den gesetzten Block, um diesen zu bearbeiten!")
        ));
        meta.setEnchantmentGlintOverride(true);
        meta.itemName(Component.text(unusedBlockID.toString()));
        item.setItemMeta(meta);
        return item;
    }

    public static @NotNull Recipe getRecipe() {
        NamespacedKey key = new NamespacedKey(PixelBlocksPlugin.plugin, "pixelblock");
        ShapedRecipe recipe = new ShapedRecipe(key, getEmptyPixelBlock());
        recipe.shape("ABA", "BCB", "ABA");
        recipe.setIngredient('A', Material.DIAMOND_BLOCK);
        recipe.setIngredient('B', Material.EMERALD_BLOCK);
        recipe.setIngredient('C', Material.GRAY_STAINED_GLASS);
        return recipe;
    }
}

