package eu.mhsl.minecraft.pixelblocks.listeners;

import eu.mhsl.minecraft.pixelblocks.PixelBlockItem;
import eu.mhsl.minecraft.pixelblocks.PixelBlocksPlugin;
import eu.mhsl.minecraft.pixelblocks.pixelblock.PixelBlock;
import eu.mhsl.minecraft.pixelblocks.pixelblock.PixelBlockWorld;
import eu.mhsl.minecraft.pixelblocks.utils.Direction;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.serializer.plain.PlainTextComponentSerializer;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.World;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.block.BlockPlaceEvent;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.UUID;

public class PlacePixelBlockListener implements Listener {
    @EventHandler
    static void onBlockPlace(BlockPlaceEvent event) {
        ItemStack usedItem = event.getItemInHand();
        ItemMeta usedItemMeta = usedItem.getItemMeta();
        Component displayName = usedItemMeta.displayName();
        if(displayName == null) return;
        if(!displayName.toString().contains("Pixelblock")) return;
        if(!usedItemMeta.getEnchantmentGlintOverride()) return;

        World playerWorld = event.getPlayer().getWorld();
        if(PixelBlockWorld.isPixelWorld(playerWorld)) {
            event.getPlayer().sendMessage("In Pixelblöcken kann kein Pixelblock erstellt werden.");
            event.setCancelled(true);
            return;
        }

        Location newBlockLocation = event.getBlock().getLocation();
        playerWorld.getBlockAt(newBlockLocation).setType(Material.AIR);

        Direction direction = Direction.vectorToDirection(event.getPlayer().getLocation().getDirection());
        String itemName = PlainTextComponentSerializer.plainText().serialize(usedItemMeta.itemName());

        PixelBlock pixelBlock;
        if(itemName.equals(PixelBlockItem.unusedBlockID.toString())) {
            pixelBlock = new PixelBlock(
                newBlockLocation,
                event.getPlayer().getUniqueId(),
                UUID.randomUUID(),
                direction
            );
        } else {
            UUID itemUUID = UUID.fromString(itemName);
            pixelBlock = PixelBlocksPlugin.pixelBlocks.stream()
                .filter(block -> block.blockUUID.equals(itemUUID))
                .findFirst()
                .orElseGet(() -> new PixelBlock(
                    newBlockLocation,
                    event.getPlayer().getUniqueId(),
                    itemUUID,
                    direction
                ));
        }

        try {
            pixelBlock.place(newBlockLocation, direction);
        } catch (IllegalArgumentException e) {
            event.setCancelled(true);
            event.getPlayer().sendMessage(Component.text(e.getMessage()));
        }
    }
}
